/*************************************************** 
 Magic Mirror control program.  
  For the Arduino Leonardo and Micro.
 
 
 
 
 
 Sends a keyboard commands based on motion sensor triggers.  
 Keyboard commands are received by  computer puppet program 
 created by Imagineerieing (http://www.imagineerieing.com).  
 Specifically written to control Yorick puppet 
 (http://www.imagineerieing.com/yorick.html). 
 
 The circuit:
 * PIR Motion sensor #1 attached from pin 4
 * PIR Motion sensor #2 attached from pin 7
 
 VS1053 - Arduino Micro
 Vcc - ICSP 5V
 GND - ICSP GND
 SCLK - ICSP SCK
 MISO - ICSP MISO
 MOSI - ICSP MOSI
 RST - Pin 9 (D9)
 CS - Pin 10 (D10)
 XCDS - Pin 8 (D8)
 SDCS - Pin 6 (D6)
 DREQ - Pin 3 (SCL, D3, INT0); must be an interrupt pins 0-3
 
 created 14 Oct 2013
 modified 29 Oct 20132
 by Krist Norlander
 
 ****************************************************/

// include SPI, MP3 and SD libraries
#include <SPI.h>
#include <Adafruit_VS1053.h>
#include <SD.h>

// define the pins used
//#define CLK 13       // SPI Clock, shared with SD card
//#define MISO 12      // Input data, from VS1053/SD card
//#define MOSI 11      // Output data, to VS1053/SD card
// Connect CLK, MISO and MOSI to hardware SPI pins. 
// See http://arduino.cc/en/Reference/SPI "Connections"

// These can be any pins:
#define RESET 9      // VS1053 reset pin (output)
#define CS 10        // VS1053 chip select pin (output)
#define DCS 8        // VS1053 Data/command select pin (output)
#define CARDCS 6     // Card chip select pin
// DREQ should be an Int pin, see http://arduino.cc/en/Reference/attachInterrupt
#define DREQ 3       // VS1053 Data request, ideally an Interrupt pin
 
//define global constants 
const char fKey      = 'f';             //key for "furious" expression
const char sKey      = 's';             //key for "sad" expression
const char dKey      = 'd';             //key for "docile" expression
const char shiftKey  = KEY_LEFT_SHIFT;  //key to open mouth
const char leftKey   = KEY_LEFT_ARROW;  //key for turning head left
const char rightKey  = KEY_RIGHT_ARROW; //key for turning head right
const char upKey     = KEY_UP_ARROW;    //key for nodding head up
const char downKey   = KEY_DOWN_ARROW;  //key for nodding head down
const char escapeKey = KEY_ESC;         //key activate options menu
const char deleteKey = KEY_DELETE;      //key to hide head
const int sensor_1_Pin  = 4;            // input pin for sensor #1 trigger
const int sensor_2_Pin  = 7;            // input pin for sensor #2 trigger

//define global variables
int previousSensor1State = HIGH;   // for checking the state of a sensor #1 trigger
int previousSensor2State = HIGH;   // for checking the state of a sensor #2 trigger
long randNum = 0.0;                 // for random number generation

//music player constructor
Adafruit_VS1053_FilePlayer musicPlayer = Adafruit_VS1053_FilePlayer(RESET, CS, DCS, DREQ, CARDCS);

//initialize program parameters
void setup() {
  Serial.begin(9600); //test - allows listen to serial comm from arduino
  delay(5000); //allow time to enable serial monitor [CTRL+SHIFT+M]

  Serial.println("Testing Magic Mirror program: ");
  Serial.println("Setup - Initialization");

  //generate a random seed
  Serial.println("Setup - Setting random seed");
  randomSeed(analogRead(0));
  
  // make the sensor trigger pins an input:
  Serial.println("Setup - Setting Sensor Pin INPUT mode");
  pinMode(sensor_1_Pin, INPUT);
  pinMode(sensor_2_Pin, INPUT);

  // set delay for user to setup computer program
  Serial.println("Setup - Delay 10 seconds for setting computer puppet program...");
  for (int i = 10; i > 0; i--) {
    Serial.print(i);
    Serial.print("...");
    delay(1000);
  }
  Serial.println("0");

  // initialize audio and SD card:
  Serial.println("Setup - Initializing audio player and SD card");
  musicPlayer.begin(); // initialise the music player
  SD.begin(CARDCS);    // initialise the SD card
  
  // Set volume for left, right channels. lower numbers == louder volume!
  Serial.println("Setup - Setting audio player volume");
  musicPlayer.setVolume(20,20);

  // Set interrupt function
  Serial.print("Setup - Setting audio player interrupt to ");
  Serial.println(VS1053_FILEPLAYER_PIN_INT);
  //musicPlayer.useInterrupt(VS1053_FILEPLAYER_TIMER0_INT); // timer int
  musicPlayer.useInterrupt(VS1053_FILEPLAYER_PIN_INT);  // DREQ int

  // initialize control over the keyboard:
  Serial.println("Setup - Initializing keyboard control");
  Keyboard.begin();  

  // hide puppet
//  Serial.println("Setup - Hiding puppet");
//  Keyboard.press(deleteKey); //DEL key = hide
  Keyboard.releaseAll();
}

//primary processing loop
void loop() {

  // read the sensor triggers:
//  Serial.print("Loop - Reading sensor state: ");
  int sensor1State = digitalRead(sensor_1_Pin);
  int sensor2State = digitalRead(sensor_2_Pin);
//  Serial.print("Sensor 1 = ");
//  Serial.print(sensor1State);
//  Serial.print(", Sensor 2 = ");
//  Serial.println(sensor2State);

/*
  //if all sensors inactive, then hide head
  if ((sensor1State == LOW) && (sensor2State == LOW)) {
    Serial.println("No sensors active - hiding head");
    Keyboard.write(deleteKey);
  }
*/

  // testing
  Serial.println("Loop - setting sensor states HIGH for testing");
  sensor1State = HIGH;
  sensor2State = HIGH;

  //if all sensors active, then say a random message
  if ((sensor1State == HIGH) && (sensor2State == HIGH)) {
//    Serial.print("Both sensors active - saying message: ");

    //generate a random number between 0 and 9 and make sure not sequential repeats
    int temp = int(random(10));      
    while (temp == randNum) temp = int(random(10));   
    randNum = temp;

//testing
randNum = 8;
//Keyboard.write('d'); //key 'd' = "docile"

    switch (randNum) {
      case 0:
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("beware.mp3");
          Serial.println("Beware of what comes next!");  
          sayPhrase(6);
        }
        break;
      case 1:
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("looking.mp3");
          Serial.println("What are you looking at?");  
          sayPhrase(5);
        }
        break;
      case 2:
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("help.mp3");
          Serial.println("Help! Get me out of here!");  
          sayPhrase(5);
        }
        break;
      case 3:
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("hair.mp3");
          Serial.println("Your hair is worse than mine!");  
          sayPhrase(5);
        }
        break;
      case 4:
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("eye.mp3");
          Serial.println("Look for my eye");  
          Keyboard.press(downKey);  //look down while talking  
          sayPhrase(4);
          Keyboard.releaseAll();
        }
        break;
      case 5:
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("wolf.mp3");
          Serial.println("Watch out for the werewolf!");  
          sayPhrase(5);
        }
        break;
      case 6:
        Serial.println("Loop - inside case 6...");  
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("scared.mp3");
          Serial.println("You look scared!");  
          sayPhrase(3);
        }
        break;
      case 7:
        Serial.println("Loop - inside case 7...");  
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("gday.mp3");
          Serial.println("You look scared!");  
          sayPhrase(5);
        }
        break;
      case 8:
        Serial.println("Loop - inside case 8...");  
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("watching.mp3");
          Serial.println("You look scared!");  
          sayPhrase(7);
        }
        break;
/*
      case 7:
        Serial.println("Loop - inside case 7...");  
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          Serial.println("Loop - playing gday.mp3 track");  
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("gday.mp3");
          Serial.print("Loop - saying message: ");  
          Serial.println("G'Day to ya Mat'y");  
          sayPhrase(5);
        }
        break;
      case 8:
        Serial.println("Loop - inside case 8...");  
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          Serial.println("Loop - playing move.mp3 track");  
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("move.mp3");
          Serial.print("Loop - saying message: ");  
          Serial.println("Move along now");  
          Keyboard.press(rightKey);  //look right while talking  
          sayPhrase(3);
          Keyboard.releaseAll();
        }
        break;
      case 9:
        Serial.println("Loop - inside case 9...");  
        // check if playing in the background
        if (! musicPlayer.playingMusic) {
          Serial.println("Loop - playing watching.mp3 track");  
          // Play another file in the background, REQUIRES interrupts!
          musicPlayer.startPlayingFile("watching.mp3");
          Serial.print("Loop - saying message: ");  
          Serial.println("I am keeping one eye on you");  
          sayPhrase(7);
        }
        break;
*/        
      default: //if nothing matches; look up and down
        Serial.println("...Default Case...");  
        Keyboard.press(upKey);    //look up for 1/2 second
        delay(500);      
        Keyboard.press(downKey);  //look down for 1/2 second  
        Keyboard.release(upKey);  //start down befor release key
        delay(500);      
        Keyboard.releaseAll();
    }
  }

/*  
  // if the sensor state has changed, and it's currently active
  if ((buttonState != previousSensor1State) && (sensor1State == HIGH)) {

    Keyboard.write('f'); //key 'f' = "furious"
    Keyboard.write('s'); //key 's' = "sad"
    Keyboard.write('d'); //key 'd' = "docile"
    if (counter == 4) {
      Keyboard.press(deleteKey); //DEL key = hide
      Keyboard.releaseAll();
    }
    if (counter == 5) Keyboard.write(115); //key 's' = "sad"
    if (counter == 6) Keyboard.write(102); //key 'f' = "furious"
    if (counter == 7) {
      Keyboard.press(shiftKey); //SHIFT key = open mouth
      delay(2000);
      Keyboard.releaseAll();
    }   
    if (counter == 8) {
      Keyboard.press(leftKey); //LEFT ARROW key = look left
      delay(2000);
      Keyboard.releaseAll();
    }

    if (counter == 9) {
      Keyboard.press(rightKey); //RIGHT ARROW key = look right
      delay(2000);
      Keyboard.releaseAll();
    }
    if (counter > 9) {
      Keyboard.press(escapeKey); //ESC key = options box
      Keyboard.releaseAll();
    }
  }

*/

  // save the current sensor states for comparison next time:
  previousSensor1State = sensor1State; 
  previousSensor2State = sensor2State; 
  
  // File is playing in the background
  if (! musicPlayer.playingMusic) 
    Serial.println("Done playing music");

    Serial.println("Loop - done with loop; begin again in 1 second...");
    Serial.println("");
    delay(2000);
}

//function to make mouth move 6 times
void sayPhrase(int num) {
//  Serial.print("Moving mouth ");  
//  Serial.print(num);  
//  Serial.println(" times");  
//  Keyboard.releaseAll();
  for (int i = 0; i < num; i++) {
    Keyboard.press(shiftKey);
    delay(140); //delay for mouth open; adjust for timing sync
    Keyboard.release(shiftKey);
    delay(140); //delay for mouth shut; adjust for timing sync
  }
  Keyboard.releaseAll();
} 


/****************

END

****************/


